from dataclasses import dataclass
from functools import cache
from pathlib import Path
import logging

from .singleton import Singleton


@dataclass
class Defaults(metaclass=Singleton):
    root: logging.Logger = logging.getLogger("manga")
    path: Path = Path("/tmp/manga_scrape.log")
    level: int = logging.INFO
    file_level: int = logging.INFO
    stream_level: int = logging.WARNING
    fmt: str = "%(asctime)s - %(levelname)s - %(module)s - %(message)s"


@cache
def get_logger(name: str) -> logging.Logger:
    d = Defaults()
    ret = d.root.getChild(name)
    ret.setLevel(d.level)
    for h, lvl in ((logging.FileHandler(d.path), d.file_level), (logging.StreamHandler(), d.stream_level)):
        h.setLevel(lvl)
        h.setFormatter(logging.Formatter(d.fmt))
        ret.addHandler(h)
    return ret



class Singleton(type):
    _objs = {}  # type: ignore

    def __call__(cls, *args, **kwargs):
        if cls not in cls._objs:
            cls._objs[cls] = super().__call__(*args, **kwargs)
        return cls._objs[cls]



from pathlib import Path
import argparse
import platform
import sys

from manga.sites import domains
from .test_sites import test_sites


class Supported(argparse.Action):
    def __call__(self, _, _2, arg, _3):
        arg = "" if arg is None else arg
        match = tuple(i for i in domains if arg in i)
        print("No matches found." if len(match) == 0 else "\n".join(match))
        sys.exit(0)


def main(prog: str, *args: str) -> bool:
    assert "Darwin" == platform.system(), "Not on Mac! Remember to change name and ext!"
    parser = argparse.ArgumentParser(prog=Path(prog).name)
    parser.add_argument("--supported", action=Supported, nargs="?")
    parser.add_argument("directory", type=Path, help="The directory to test")
    parser.add_argument("--skip", type=str, nargs="+", default=[], help="Domains to skip")
    parser.add_argument(
        "--skip-tiny", action="store_true", help="Do not open sites that were not tested because of a low chapter count"
    )
    parser.add_argument(
        "--skip-point-five", action="store_true", help="Do not open sites have a .5 chapter after the latest chapter"
    )
    parser.add_argument("--opener", default="open", help="The default binary to open a URL with")
    parser.add_argument("--no-prompt", action="store_true", help="Auto open sites when complete, do not prompt user")
    parser.add_argument(
        "--delay",
        default=0,
        type=int,
        help="The number of seconds each thread should wait between testing sites (to avoid DOSing)",
    )
    return test_sites(**vars(parser.parse_args(args)))


def cli() -> None:
    sys.exit(0 if main(*sys.argv) else -1)
